$PBKfile = $env:userprofile + "\AppData\Roaming\Microsoft\Network\Connections\Pbk\rasphone.pbk"
# for testing purposes you can specify another filename
$PBKfileNew = $PBKfile

$ScriptPath = $MyInvocation.MyCommand.Name | Split-Path -resolve
$ConfigurationFile = "$ScriptPath\AOVPN-rasphone.pbk"
$VPNName = "Always On VPN"
$MyEncoding = "ascii"

#If you need to see verbose informations set the value only to "Continue"
$VerbosePreference = "SilentlyContinue"
$ErrorActionPreference = "SilentlyContinue"

function Parse-PBKFile {
    param (
        [array] $FileContent,
        [array] $Configs
    )
    $NameLines = @{ }

    # walk through all lines from the file
    For ($i = 0; $i -lt $FileContent.count; $i++) {
        If ($FileContent[$i] -match "^\[(.*)\]$") {
            $NameLines.Add($i, $matches[1])
        }
    }
    Write-Output $NameLines
}

function Modify-VPNConfiguration {
    param (
        $FileContent,
        $Configs
    )
    [int] $ConfigStart = 0
    [int] $ConfigEnd = 0

    If ($Configs.ContainsValue($VPNName)) {
        $ConfigStart = ($Configs.GetEnumerator() | Where-Object { $_.Value -eq $VPNName }).Name
        $ConfigEnd = FindEndLine -Configs $Configs -StartLine $ConfigStart
        If ($ConfigEnd -eq 0) { $ConfigEnd = $FileContent.Count }
    }
    Else {
        $ConfigStart = $FileContent.Count
        $ConfigEnd = $FileContent.Count
    }

    Write-Verbose "Config start: $ConfigStart"
    Write-Verbose "Config end: $ConfigEnd"
    Write-Verbose "File end: $($FileContent.Count)"
    Add-NewProfile -StartLine $ConfigStart -EndLine $ConfigEnd -FileContent $FileContent
}

function FindEndLine {
    param (
        [int] $StartLine,
        $Configs
    )
    $Lines = $Configs.Keys | Sort-Object
    Foreach ($Id in $Lines) {
        If ($Id -gt $StartLine) {
            Return $Id
        }
    }
    return 0
}

function Add-NewProfile {
    param (
        [int] $StartLine,
        [int] $EndLine,
        $FileContent
    )

    # Create empty file
    New-Item -Path $PBKfileNew -ItemType File -Force | Out-Null

    $VPNProfile = Get-Content -Path $ConfigurationFile

    # beginning of the file
    For ($i = 0; $i -lt $StartLine; $i++) {
        $FileContent[$i] | Out-File -Append -Encoding $MyEncoding -FilePath $PBKfileNew
    }
    # content after bypassed VPN profile
    For ($i = $EndLine; $i -lt $FileContent.count; $i++) {
        $FileContent[$i] | Out-File -Append -Encoding $MyEncoding -FilePath $PBKfileNew
    }
    # add the new profile
    $VPNProfile | ForEach-Object { $_ | Out-File -Append -Encoding $MyEncoding -FilePath $PBKfileNew }

    # add ending line
    "" | Out-File -Append -Encoding $MyEncoding -FilePath $PBKfileNew
}

# Start of script -----------------------------------------------

# setup event logging
$EventLogSource = "Always On VPN"
New-EventLog -LogName Application -Source $EventLogSource
Write-EventLog -LogName Application -Source $EventLogSource -EntryType Information -EventID 1000 -Message "Script starts"

try {
    $FileContent = Get-Content -Path $PBKfile
}
catch {
    # Create empty file
    New-Item -Path $PBKfileNew -ItemType File -Force | Out-Null
    $FileContent = ""
}

$Configs = Parse-PBKFile -FileContent $FileContent
if ($Configs) { Write-EventLog -LogName Application -Source $EventLogSource -EntryType Information -EventID 1001 -Message "PBK file parsed" }
Modify-VPNConfiguration -FileContent $FileContent -Configs $Configs
Write-EventLog -LogName Application -Source $EventLogSource -EntryType Information -EventID 1002 -Message "PBK file modified"